@@ -78,9 +78,11 @@ impl Editor {
             if let Ok(doc) = doc {
                 doc
             } else {
-                initial_status.text = format!("ERR: Could not open file: {}", file_name);
+                initial_status.text = format!("ERR: Could not open file (opening as new): {}", file_name);
                 initial_status.color = Option::from(crate::ERROR_COLOR);
-                Document::default()
+                let mut doc = Document::default();
+                doc.set_file_name(file_name.to_owned());
+                doc
             }
         } else {
             Document::default()
@@ -270,7 +272,7 @@ impl Editor {
             Key::Right => {
                 if x < width {
                     x = x.saturating_add(1);
-                } else if y < height {
+                } else if y < height - 1 {
                     y = y.saturating_add(1);
                     x = 0;
                 }
@@ -314,7 +316,8 @@ impl Editor {
     pub fn draw_row(&self, row: &Row) {
         let width = usize::from(self.terminal.size().width);
         let start = self.offset.x;
-        let end = self.offset.x.saturating_add(width);
+        #[allow(clippy::integer_arithmetic)]
+        let end = self.offset.x + width - self.terminal.size().height.to_string().len();
         let row = row.render(start, end);
         println!("{}\r", row);
     }
